/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: DacConfig.java,v 1.2 2005/09/07 13:38:06 rampil Exp $
 */
package org.logical_paradox.vsan.dac;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Properties;

import org.logical_paradox.common.cli.Argument;
import org.logical_paradox.common.cli.ArgumentParser;
import org.logical_paradox.common.util.StringUtils;
import org.logical_paradox.vsan.*;

/**
 * fBXNACRg[[̃RtBO[V
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.2 $
 */
public class DacConfig {
	/** RtBOt@C */
	public static final String OPT_CONFIG = "config";

	/** ݒl - |[gԍ */
	public static final String KEY_PORT = "port";
	/** ݒl - ڑ */
	public static final String KEY_CONNECTIONS = "connections";
	/** ݒl - Mobt@TCY */
	public static final String KEY_RECVSIZE = "recvsize";
	/** ݒl - f[^x[Xt@C */
	public static final String KEY_DATABASE_DIR = "database-dir";
	/** ݒl - m[hID */
	public static final String KEY_NODE_ID = "node-id";
	/** ݒl - NɃhCɐڑ邩ǂ */
	public static final String KEY_JOIN_ON_STARTUP = "join-on-startup";
	/** ݒl - hCRg[[ */
	public static final String KEY_DOMAIN_CONTROLLER = "domain-controller";

	/** ftHg̐ݒt@C */
	public static final String DEFAULT_CONFIG_FILENAME = "conf/dac.conf";
	/** ftHg̃XjO|[gԍ */
	public static final int DEFAULT_PORT_NUMBER = 9876;
	/** ftHg̃RlNV */
	public static final int DEFAULT_CONNECTIONS = 25;
	/** ftHg̎Mobt@TCY */
	public static final int DEFAULT_RECV_SIZE = 4096;
	/** ftHg̃f[^x[XfBNg */
	public static final String DEFAULT_DATABASE_DIR = "./database";
	/** ftHg̋Nݒ - NɃhCɐڑ邩ǂ */
	public static final boolean DEFAULT_JOIN_ON_STARTUP = true;

	/** RtBOt@C */
	private String config = DEFAULT_CONFIG_FILENAME;
	/** DACG[WFNglisten|[gԍ */
	private int port = DEFAULT_PORT_NUMBER;
	/** őRlNV */
	private int connections = DEFAULT_CONNECTIONS;
	/** Mobt@TCY */
	private int recvsize = DEFAULT_RECV_SIZE;
	/** f[^x[XfBNg */
	private String databaseDir = DEFAULT_DATABASE_DIR;
	/** m[hID */
	private String nodeId;
	/** NɃhCɐڑ邩ǂ */
	private boolean joinOnStartup = DEFAULT_JOIN_ON_STARTUP;
	/** hCRg[[ */
	private String domainController = null;

	/**
	 * RXgN^
	 * @param args 
	 */
	public DacConfig(String[] args) throws FileNotFoundException, ConfigException, IOException {
		parse(args);
	}

	/**
	 * ݒt@CԂ
	 * @return ݒt@C
	 */
	public String getConfigFilename() {
		return config;
	}
	/**
	 * zXgAhXԂD
	 * @return zXgAhX
	 */
	public String getHost() {
		try {
			String hostname = InetAddress.getLocalHost().toString();
			if(hostname.indexOf("/") < 0) {
				return hostname;
			} else {
				return hostname.split("/")[1];
			}
		} catch (UnknownHostException e) {
			return "localhost";
		}
	}
	/**
	 * DACG[WFglisteningĂ|[gԍԂ
	 * @return |[gԍ
	 */
	public int getPort() {
		return port;
	}
	/**
	 * m[hIDԂ
	 * @return m[hID
	 */
	public String getNodeId() {
		return nodeId;
	}
	/**
	 * ڑԂ
	 * @return ڑ
	 */
	public int getConnections() {
		return connections;
	}
	/**
	 * Mobt@̃TCYԂ
	 * @return Mobt@̃TCY
	 */
	public int getRecvsize() {
		return recvsize;
	}
	/**
	 * f[^x[XfBNgԂ
	 * @return f[^x[XfBNg
	 */
	public String getDatabaseDir() {
		return databaseDir;
	}
	/**
	 * NɃhCɐڑ邩ǂԂD
	 * @return true:ڑ / false:ڑȂ
	 */
	public boolean isJoinOnStartup() {
		return joinOnStartup;
	}
	/**
	 * hCRg[[ԂD
	 * @return hCRg[[
	 */
	public String getDomainController() {
		return domainController;
	}
	/**
	 * w肳ꂽIvV荞
	 * @param args 
	 * @throws ConfigException 荞݂ɎsꍇɔO
	 */
	protected void parse(String[] args) throws ConfigException, FileNotFoundException, IOException {
		Argument[] arguments = ArgumentParser.parse(args);

		for(int i = 0; i < arguments.length; i++) {
			Argument arg = arguments[i];
			if(arg.isOption() == true) {
				if(OPT_CONFIG.equals(arg.getName())) {
					config = arg.getValue();
				} else {
					throw new ConfigException("sȃIvVw肳܂:" + arg.getName());
				}
			} else {
				throw new ConfigException("sȈw肳܂:" + arg.getValue());
			}
		}

		// RtBOt@C[h
		Properties p = loadConfigration();

		// vpeBl荞
		// 'port'
		if(StringUtils.isEmpty(p.getProperty(KEY_PORT)) == false) {
			port = Integer.parseInt(p.getProperty(KEY_PORT));
		}
		// 'connections'
		if(StringUtils.isEmpty(p.getProperty(KEY_CONNECTIONS)) == false) {
			connections = Integer.parseInt(p.getProperty(KEY_CONNECTIONS));
		}
		// 'recvsize'
		if(StringUtils.isEmpty(p.getProperty(KEY_RECVSIZE)) == false) {
			recvsize = Integer.parseInt(p.getProperty(KEY_RECVSIZE));
		}
		// 'database-filename'
		if(StringUtils.isEmpty(p.getProperty(KEY_DATABASE_DIR)) == false) {
			databaseDir = p.getProperty(KEY_DATABASE_DIR);
		}
		// 'node-id'
		if(StringUtils.isEmpty(p.getProperty(KEY_NODE_ID)) == true) {
			throw new ConfigException("node-id͕K{ł");
		} else {
			nodeId = p.getProperty(KEY_NODE_ID);
		}
		// 'join-on-startup'
		if(StringUtils.isEmpty(p.getProperty(KEY_JOIN_ON_STARTUP)) == false &&
			"true".equals(p.getProperty(KEY_JOIN_ON_STARTUP).toLowerCase()) == true
		) {
			joinOnStartup = true;
		} else {
			joinOnStartup = false;
		}
		// 'domain-controller'
		if(joinOnStartup == true) {
			if(StringUtils.isEmpty(p.getProperty(KEY_DOMAIN_CONTROLLER)) == true) {
				throw new ConfigException("join-on-startupݒ莞Cdomain-controller͕K{ł");
			} else {
				domainController = p.getProperty(KEY_DOMAIN_CONTROLLER);
			}
		}
	}
	/**
	 * RtBOt@C[h
	 * @return vpeBƂă[hꂽl
	 */
	protected Properties loadConfigration() throws FileNotFoundException, IOException {
		Properties p = new Properties();
		p.load(new FileInputStream(getConfigFilename()));

		return p;
	}
}
