/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: Listener.java,v 1.3 2005/09/17 17:35:49 rampil Exp $
 */
package org.logical_paradox.vsan.dac;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.logical_paradox.common.net.ReadWriteBuffer;
import org.logical_paradox.common.session.IllegalSessionStateException;
import org.logical_paradox.common.session.Session;
import org.logical_paradox.common.util.ByteStreamConverter;
import org.logical_paradox.vsan.dac.command.CommandFactory;
import org.logical_paradox.vsan.dac.command.exception.CommandExecutionException;
import org.logical_paradox.vsan.dac.command.exception.SessionTerminationException;
import org.logical_paradox.vsan.dac.command.exception.ShutdownException;
import org.logical_paradox.vsan.dac.command.handler.DacCommand;
import org.logical_paradox.vsan.node.DatabaseAccessException;

/**
 * Xi[
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.3 $
 */
public class Listener extends Thread {
	/** K[ */
	private static final Log log = LogFactory.getLog(Listener.class);

	/** fBXNACRg[[{ */
	private final DiskArrayController dac;
	/** DACRtBO[V */
	private final DacConfig config;
	/** NCAg\Pbg */
	private Socket clientSocket;
	/** ZbV */
	private Session session;
	/** bNIuWFNg */
	private Object lock = new Object();
	/** Xi[ԍ */
	private final int listenerNo;
	/** Mobt@ */
	private ReadWriteBuffer recv;
	/** m[hReLXg */
	private final NodeContext context;

	/**
	 * RXgN^
	 * @param parent eXbh
	 * @param config_ DACRtBO[V
	 * @param no Xi[ԍ
	 */
	public Listener(DiskArrayController parent, NodeContext context_, int no) {
		context = context_;
		config = context_.getConfig();
		listenerNo = no;
		recv = new ReadWriteBuffer(config.getRecvsize());
		dac = parent;
	}
	/**
	 * NCAg\Pbg̃Xi[ň悤ɐݒ肷
	 * @param client_ NCAg\Pbg
	 */
	public void accept(Socket client_) {
		// NCAg\Pbgݒ肵ăbN
		synchronized(lock) {
			clientSocket = client_;
			lock.notifyAll();
		}
	}
	/**
	 * Xi[ԍԂ
	 * @return Xi[ԍ
	 */
	public int getListenerNo() {
		return listenerNo;
	}
	/**
	 * XbhC
	 */
	public void run() {
		try {
			while(true) {
				synchronized(lock) {
					// NCAg\Pbgڑ҂
					while(clientSocket == null) {
						lock.wait();
					}

					// NCAg̃f[^M
					try {
						while(true) {
							if(session == null) {
								// ܂݂ȂȂCVZbVJn
								session = DiskArrayController.getSessionManager().getSession();
								// NCAg\Pbgݒ肷
								session.setAttribute(DacConstant.SKEY_SOCKET_INPUT_STREAM, new BufferedInputStream(clientSocket.getInputStream()));
								// m[hReLXgݒ肷
								session.setAttribute(DacConstant.SKEY_NODE_CONTEXT, context);
							}
							// 1oCg擾
							InputStream in = (InputStream)session.getAttribute(DacConstant.SKEY_SOCKET_INPUT_STREAM);
							int b = in.read();

							if(b == -1) {
								// ڑN[Yꂽ
								throw new IOException();
							}

							DacCommand command = CommandFactory.getCommand(b);
							if(command == null) {
								// w肳ꂽ^CṽR}h݂Ȃ̂ŃG[ƂĖ
								log.warn("w^Cv[" + b + "]ɑΉR}h`ł");
								continue;
							}

							// R}h̎s
							boolean execSuccess = false;
							try {
								session = command.execute(session);
								// X|Xꍇ́Ce𑗐M
								Response response = (Response)session.getAttribute(DacConstant.SKEY_DAC_RESPONSE);
								if(response != null) {
									sendResponse(response);
								}
								execSuccess = true;
							} catch (CommandExecutionException e) {
								log.error("R}hsɗO܂", e);
								sendResponse(new Response(ResponseCode.COMMAND_EXECUTION_EXCEPTION, new byte[0]));
							} catch (DatabaseAccessException e) {
								log.error("f[^x[XANZXQ", e);
								sendResponse(new Response(e.getErrorCode(), new byte[0]));
							} finally {
								// \Pbg瑗MĂāAM̃f[^SĎ̂Ă
								// AG[ăS~cĂ܂ꍇ̂
								if(execSuccess == false) {
									int remains = in.available();
									for(int i = 0; i < remains; i++) {
										in.read();
									}
								}
								// ZbVC񓚃f[^폜
								session.removeAttribute(DacConstant.SKEY_DAC_RESPONSE);
							}
						}
					} catch (IOException e) {
						closeSession();
						lock.notifyAll();
					} catch(SessionTerminationException e) {
						closeSession();
						lock.notifyAll();

						// Vbg_E̎w߂łꍇCDAC̃Xe[^XuIvɐݒ肷
						if(e instanceof ShutdownException) {
							dac.setServerStatus(DiskArrayController.E_CLOSING);
						}
					} catch (IllegalSessionStateException e) {
						log.error("VKZbV̎擾Ɏs܂");
						closeSession();
						lock.notifyAll();
					}
				}
			}
		} catch(InterruptedException e) {
			// tearDown()OR[ƁCɏ
			log.info("Xi[" + listenerNo + "~܂");
		}
	}
	/**
	 * NCAgɑ΂DAC̃X|Xԋp
	 * @param data Mf[^
	 */
	protected void sendResponse(Response response) throws IOException {
		BufferedOutputStream bo = new BufferedOutputStream(clientSocket.getOutputStream());

		// ܂CX|XR[h𑗐M
		bo.write(ByteStreamConverter.convert(response.getResponseCode()));

		byte[] data = response.getData();

		int packetCount = (data.length / DacConstant.PACKET_SIZE) + 1;
		int begin = 0;

		// ɗ\pPbg𑗐M
		byte[] packetnum = ByteStreamConverter.convert(packetCount);
		bo.write(packetnum);

		// pPbg𑗐M
		for(int i = 0; i < packetCount; i++) {
			int len = DacConstant.PACKET_SIZE;
			if(begin + len >= data.length) {
				len = data.length - begin;
			}
			byte[] datalen = ByteStreamConverter.convert(len);
			// pPbgTCY̑M
			bo.write(datalen);
			// f[^ubN̑M
			bo.write(data, begin, len);
		}
		bo.flush();
	}
	/**
	 * NCAgƂ̐ڑIɐؒf
	 * o͗OȂǂĂCSĖ
	 */
	protected void closeSession() {
		log.info("ZbVN[Y܂");
		try {
			if(clientSocket != null) {
				clientSocket.close();
			}
		} catch (IOException e) {
			// ؒfɉ炩̓o͗OC(ɃNCAgcloseꍇȂ)
		} finally {
			clientSocket = null;
			// ZbVIɃN[Y
			if(session != null) {
				DiskArrayController.getSessionManager().removeSession(session.getSessionId());
				session = null;
			}
			// EFCeBOXgɒǉ
			dac.addWaitingThread(this);
		}
	}
	/**
	 * Xi[Vbg_E
	 */
	protected void tearDown() {
		log.info("Xi[" + listenerNo + "AVbg_EvM܂");
		closeSession();

		interrupt();
	}
}
