/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: DomainFileSystemImpl.java,v 1.2 2005/09/07 13:38:06 rampil Exp $
 */
package org.logical_paradox.vsan.dfs;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.logical_paradox.vsan.dfs.event.DomainUpdateEventObject;
import org.logical_paradox.vsan.dfs.event.DomainUpdateListener;
import org.logical_paradox.vsan.domain.Domain;
import org.logical_paradox.vsan.domain.DomainController;
import org.logical_paradox.vsan.lookup.LookupFactory;

/**
 * hCt@CVXe̎
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.2 $
 */
public class DomainFileSystemImpl extends UnicastRemoteObject implements DomainFileSystem, DomainUpdateListener {
	/** K[ */
	private static final Log log = LogFactory.getLog(DomainFileSystemImpl.class);

	/** ̃hCt@CVXẽRtBO[V */
	private final DfsConfig config;
	/** m[hID */
	private final String nodeId;

	/** hCRg[[ */
	private DomainController domainController;

	/** hC */
	private Domain domain;

	/** hC񎩓XVXbh */
	private DomainSynchroThread domainSynchronizer;

	/** NCAgڑj^[Xbh */
	private ClientConnectionMonitor clientMonitor;

	/** hCڑvoC_[ */
	private DomainConnectionProvider domainConnectionProvider;

	/** hCڑvLV */
	private final ArrayList proxies = new ArrayList();

	/**
	 * RXgN^
	 * @param config RtBO
	 * @throws RemoteException CX^XɎs
	 */
	public DomainFileSystemImpl(DfsConfig c) throws RemoteException {
		config = c;
		// ̂ƂAm[hID = ڑĂhCƂ
		nodeId = config.getDomainPreconnectId();

		// 
		init();
	}

	/**
	 * ڑm
	 */
	protected void init() throws RemoteException {
		log.info("hCt@CVXeĂ܂");

		// hC擾
		domainController = LookupFactory.getLookup().lookupDomainController(config.getDomainControllerId());
		domain = domainController.getDomain(config.getDomainPreconnectId());

		// ڑvoC_[쐬
		log.info("hCڑvoC_[쐬Ă܂");
		domainConnectionProvider = new DomainConnectionProvider(config, domain);

		if(config.getDomainSynchroInterval() > 0) {
			domainSynchronizer = new DomainSynchroThread(config, domainController, domain);
			domainSynchronizer.addListener(this);
			domainSynchronizer.start();
			log.info("hC񎩓XVXbh쐬܂");
		}

		if(config.getClientMonitorInterval() > 0) {
			clientMonitor = new ClientConnectionMonitor(config.getClientMonitorInterval());
			clientMonitor.start();
			log.info("NCAgڑj^[쐬܂");
		}
		log.info("hCt@CVXe܂");
	}
	/**
	 * hCύXʒmCxgD
	 * @param e hCύXIuWFNg
	 */
	public void domainChanged(DomainUpdateEventObject e) {
		log.info("hCɕύX߁Am[hŐV܂");
		ArrayList updateInfo = new ArrayList();
		List before = new ArrayList(e.getDomainBefore().getNodeList().keySet());
		List after = new ArrayList(e.getDomainAfter().getNodeList().keySet());

		// hC痣Em[h𒲂ׂ
		for(Iterator it = before.iterator(); it.hasNext();) {
			String node = (String)it.next();
			if(after.contains(node) == false) {
				// ȑÔɍ͂Ȃ = Em[h
				log.info("m[h[" + node + "]E܂");
				updateInfo.add(new DomainUpdateDetail(DomainUpdateDetail.UPDATE_TYPE_LEAVE, node));			
			}
		}
		// VhCɎQm[h𒲂ׂ
		for(Iterator it = after.iterator(); it.hasNext();) {
			String node = (String)it.next();
			if(before.contains(node) == false) {
				// ȑOȂ̂ɍ͂ = ǉꂽm[h
				log.info("m[h[" + node + "]Q܂");
				updateInfo.add(new DomainUpdateDetail(DomainUpdateDetail.UPDATE_TYPE_JOIN, node));
			}
		}

		// ̍XVC̃hCt@CVXeɐڑĂ
		// NCAgɑ΂Ēʒm
		notifyChange(updateInfo);

		log.info("hCXV܂");
	}
	/**
	 * hCXVChCt@CVXeɐڑĂNCAgɒʒmD
	 * @param changes XVẽXg
	 */
	protected void notifyChange(List changes) {
		log.info("hC̍XVeɂāANCAgɒʒm܂");
		DomainConnectionProxyImpl[] localProxies = (DomainConnectionProxyImpl[])proxies.toArray(new DomainConnectionProxyImpl[0]);

		for(Iterator it = changes.iterator(); it.hasNext();) {
			DomainUpdateDetail dud = (DomainUpdateDetail)it.next();

			if(dud.getType() == DomainUpdateDetail.UPDATE_TYPE_JOIN) {
				// ǉ
				for(int i = 0; i < localProxies.length; i++) {
					localProxies[i].getConnection().join(dud.getNode());
				}
			} else if(dud.getType() == DomainUpdateDetail.UPDATE_TYPE_LEAVE) {
				// 폜
				for(int i = 0; i < localProxies.length; i++) {
					localProxies[i].getConnection().leave(dud.getNode());
				}
			}
		}
		log.info("hC̍XVeɂāANCAgɒʒm܂");
	}
	/**
	 * hCt@CVXeɑ΂ăANZX邽߂̐ڑIuWFNgԂ
	 * ڑ̏ԂɂẮA擾łȂꍇ
	 * @param clientNodeId NCAgm[hID
	 * @return 擾łhCڑIuWFNg
	 * @throws RemoteException 擾Ƀlbg[NQȂǂ
	 */
	public DomainConnectionProxy getConnection(String clientNodeId) throws RemoteException {
		// ̃NCAgɊ蓖ĂꂽhCڑIuWFNgԂ
		// ڑɂẮA蓖ĂłȂꍇ
		DomainConnection connection = domainConnectionProvider.getConnection(clientNodeId);
		DomainConnectionProxy proxy = new DomainConnectionProxyImpl(connection, clientNodeId, config);

		proxies.add(proxy);

		return proxy;
	}
	/**
	 * ̃hCt@CVXẽm[hIDԂ
	 * @return T[o[CX^XɊ蓖Ăꂽm[hID
	 * @throws RemoteException ʐMQȂ
	 */
	public String getNodeId() throws RemoteException {
		return nodeId;
	}

	/**
	 * C\bh
	 * @param args vO
	 * @throws Exception ȂO
	 */
	public static final void main(String[] args) throws Exception {
		log.info("hCt@CVXeJnĂ܂..");

		/** \[X̓ǂݍ */
		ResourceBundle resource = ResourceBundle.getBundle(DfsConstant.RESOURCE_PREFIX);
		DfsConfig config = new DfsConfig(resource);

		log.info("T[o[RtBO[V̓ǂݍݏI.");

		// T[o[CX^X̍쐬
		DomainFileSystem filesystem = new DomainFileSystemImpl(config);

		// RMIWXgɃoCh
		Naming.rebind(filesystem.getNodeId(), filesystem);

		log.info("hCt@CVXe[" + filesystem.getNodeId() + "]Jn܂.");
	}

	/**
	 * hCXV
	 * @author satoshi akabane@logical-paradox.org
	 * @version $Revision: 1.2 $
	 */
	class DomainUpdateDetail {
		/** XV敪 - VKQ */
		public static final int UPDATE_TYPE_JOIN = 1;
		/** XV敪 - E */
		public static final int UPDATE_TYPE_LEAVE = 2;

		/** XV敪 */
		private final int type;
		/** Ώۃm[h */
		private String node;

		/**
		 * RXgN^
		 * @param type_ XV敪
		 * @param node_ m[h
		 */
		public DomainUpdateDetail(int type_, String node_) {
			type = type_;
			node = node_;
		}
		/**
		 * XV敪ԂD
		 * @return XV敪
		 */
		public int getType() {
			return type;
		}
		/**
		 * m[hԂD
		 * @return m[h
		 */
		public String getNode() {
			return node;
		}
	}
	/**
	 * hCRg[[ɒIɃANZXAm[hXVXbh
	 * @author satoshi akabane@logical-paradox.org
	 * @version $Revision: 1.2 $
	 */
	class DomainSynchroThread extends Thread {
		/** K[ */
		private final Log log = LogFactory.getLog(DomainSynchroThread.class);

		/** hCt@CVXẽRtBO[V */
		private final DfsConfig config;
		/** ڑς݃hCRg[[ */
		private final DomainController dc;
		/** ݂̃hC */
		private Domain domain = null;
		/** hCXVɌĂяo郊Xi[ */
		private final ArrayList listeners = new ArrayList();

		/**
		 * RXgN^
		 * @param config_ hCt@CVXẽRtBO[V
		 * @param dc_ ڑς݃hCRg[[
		 */
		public DomainSynchroThread(DfsConfig config_, DomainController dc_, Domain domain_) {
			config = config_;
			dc = dc_;
			domain = domain_;
			setDaemon(true);
		}
		/**
		 * XbhC
		 */
		public void run() {
			try {
				while(true) {
					sleep(config.getDomainSynchroInterval());

					// hC擾
					try {
						log.info("hC擾Ă܂");
						Domain newDomain = domainController.getDomain(config.getDomainPreconnectId());
						if(domain.hashCode() != newDomain.hashCode()) {
							// nbVR[hω̂ŁAhC񂪍XVꂽ
							DomainUpdateEventObject o = new DomainUpdateEventObject(this, domain, newDomain);
							executeCallbacks(o);

							// hCXV
							domain = newDomain;
						} else {
							log.info("hC͍XVĂ܂");
						}
					} catch (RemoteException e1) {
						// 炩̗RŁAhCRg[[hC擾łȂ
						// ꎞIȂƂȂ̂ŁA
						log.warn("hCRg[[Ƃ̒ʐMɎs܂F", e1);
						log.warn("XVXLbv܂");
						e1.printStackTrace();
					}
				}
			} catch(InterruptedException e) {
				log.info("*** hCXVXbh~܂");
			}
		}
		/**
		 * hCXVXbh~
		 */
		public void shutdown() {
			interrupt();
		}
		/**
		 * CxgXi[ND
		 * @param o CxgIuWFNg
		 */
		protected void executeCallbacks(DomainUpdateEventObject o) {

			for(Iterator it = listeners.iterator(); it.hasNext();) {
				DomainUpdateListener lsnr = (DomainUpdateListener)it.next();
				lsnr.domainChanged(o);
			}
		}
		/**
		 * hCXVʒmXi[ǉD
		 * @param lsnr hCXVʒmXi[
		 */
		public void addListener(DomainUpdateListener lsnr) {
			listeners.add(lsnr);
		}
	}
	/**
	 * NCAgڑj^[Xbh
	 * @author satoshi akabane@logical-paradox.org
	 * @version $Revision: 1.2 $
	 */
	class ClientConnectionMonitor extends Thread {
		/** K[ */
		private final Log log = LogFactory.getLog(ClientConnectionMonitor.class);

		/** NCAgڑĎԊu(msec) */
		private final long monitoringInterval;

		/**
		 * RXgN^
		 * @param interval NCAgڑĎԊu(msec)
		 */
		public ClientConnectionMonitor(long interval) {
			monitoringInterval = interval;
			setDaemon(true);
		}
		public void run() {
			try {
				while(true) {
					sleep(monitoringInterval);

					ArrayList finalist = new ArrayList();

					synchronized(proxies) {
						for(Iterator it = proxies.iterator(); it.hasNext();) {
							DomainConnectionProxyImpl proxy = (DomainConnectionProxyImpl)it.next();
							if(proxy.isExpired() == true || proxy.isGarbageTarget() == true) {
								finalist.add(proxy);
							}
						}

						if(finalist.size() > 0) {
							log.info("S[" + finalist.size() + "]̃hCڑvLVj܂");

							// ΏۂSč폜
							for(Iterator it = finalist.iterator(); it.hasNext();) {
								DomainConnectionProxyImpl proxy = (DomainConnectionProxyImpl)it.next();
								proxies.remove(proxy);
								proxy.unshare();
							}
						}
					}
				}
			} catch(InterruptedException e) {
				log.info("NCAgڑj^[~܂");
			}
		}
	}
}
